#! /usr/bin/env python
# -*- coding: utf-8 -*-
import us
import collections
import pandas as pd
import geopandas as gpd
from urllib.request import urlretrieve
from census_map_downloader.base import BaseDownloader

# Logging
import logging
logger = logging.getLogger(__name__)


class ZctasDownloader2018(BaseDownloader):
    """
    Download 5-digit ZIP Code Tabulation Area
    """
    YEAR = 2018
    # Docs pg 62 https://www2.census.gov/geo/pdfs/maps-data/data/tiger/tgrshp2018/TGRSHP2018_TechDoc_Ch3.pdf
    PROCESSED_NAME = f"zctas_{YEAR}"
    FIELD_CROSSWALK = collections.OrderedDict({
        "GEOID10": "geoid",
        "geometry": "geometry"
    })

    @property
    def url(self):
        return "https://www2.census.gov/geo/tiger/TIGER2018/ZCTA5/tl_2018_us_zcta510.zip"

    @property
    def zip_name(self):
        return f"tl_2018_us_zcta510.zip"

        # Add a raw download path for the relationships files
    @property
    def relationship_name(self):
        return "zcta_county_rel_10.txt"

    @property
    def relationship_path(self):
        return self.raw_dir.joinpath(self.relationship_name)

    def download(self):
        # Do the regular download
        super().download()

        # Also download the relationship file that connects ZCTAs to counties
        if self.relationship_path.exists():
            logger.debug(f"Relationship file already exists at {self.relationship_path}")
            return
        self.relationship_url = "https://www2.census.gov/geo/docs/maps-data/data/rel/zcta_county_rel_10.txt"
        logger.debug(f"Downloading {self.relationship_url} to {self.relationship_path}")
        urlretrieve(self.relationship_url, self.relationship_path)

    def process(self):
        """
        Refine the raw data and convert it to our preferred format, GeoJSON.
        """
        # Read in the raw shape
        gdf = gpd.read_file(self.shp_path)

        # Trim it down to the subset of fields we want to keep
        trimmed = gdf[list(self.FIELD_CROSSWALK.keys())]

        # Rename the fields using the crosswalk
        trimmed.rename(columns=self.FIELD_CROSSWALK, inplace=True)

        # Read relationship file
        df = pd.read_csv(
            self.relationship_path,
            usecols=['ZCTA5', 'STATE'],
            dtype={'ZCTA5': str, 'STATE': str}
        )

        # Loop through the 50 states
        for state in us.STATES:
            # Filter down to the ZCTAs in this state
            state_df = trimmed[trimmed.geoid.isin(df.loc[df.STATE == state.fips, 'ZCTA5'])]

            # Set page for this GeoJSON
            state_geojson_path = self.processed_dir.joinpath(f"{self.PROCESSED_NAME}_{state.abbr.lower()}.geojson")

            # Check if the geojson file already exists
            if state_geojson_path.exists():
                logger.debug(f"GeoJSON file already exists at {state_geojson_path}")
                continue

            # Write it out as GeoJSON
            logger.debug(f"Writing out {len(state_df)} shapes in {state} to {state_geojson_path}")
            state_df.to_file(state_geojson_path, driver="GeoJSON")



from .base import BaseDownloader
from .geotypes.counties import CountiesDownloader2018
from .geotypes.places import PlacesDownloader2018
from .geotypes.tracts import TractsDownloader2010
from .geotypes.zctas import ZctasDownloader2018
from .geotypes.blocks import BlocksDownloader2018


__all__ = (
    "BaseDownloader",
    "CountiesDownloader2018",
    "PlacesDownloader2018",
    "TractsDownloader2010",
    "ZctasDownloader2018",
    "BlocksDownloader2018"
)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Command-line interface.
"""
import click
import census_map_downloader


@click.group(help="Easily download U.S. census maps")
@click.option(
    "--data-dir",
    nargs=1,
    default="./",
    help="The folder where you want to download the data"
)
@click.pass_context
def cmd(ctx, data_dir="./"):
    ctx.ensure_object(dict)
    ctx.obj['data_dir'] = data_dir


@cmd.command(help="Download places")
@click.pass_context
def places(ctx):
    obj = census_map_downloader.PlacesDownloader2018(data_dir=ctx.obj['data_dir'])
    obj.run()


@cmd.command(help="Download tracts")
@click.pass_context
def tracts(ctx):
    obj = census_map_downloader.TractsDownloader2010(data_dir=ctx.obj['data_dir'])
    obj.run()


@cmd.command(help="Download counties")
@click.pass_context
def counties(ctx):
    obj = census_map_downloader.CountiesDownloader2018(data_dir=ctx.obj['data_dir'])
    obj.run()


@cmd.command(help="Download ZCTAs")
@click.pass_context
def zctas(ctx):
    obj = census_map_downloader.ZctasDownloader2018(data_dir=ctx.obj['data_dir'])
    obj.run()


@cmd.command(help="Download blocks")
@click.pass_context
def blocks(ctx):
    obj = census_map_downloader.BlocksDownloader2018(data_dir=ctx.obj['data_dir'])
    obj.run()


if __name__ == '__main__':
    cmd()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import time
import us
import pathlib
import zipfile
import geopandas as gpd
from urllib.request import urlretrieve

# Logging
import logging
logger = logging.getLogger(__name__)


class BaseDownloader(object):
    THIS_DIR = pathlib.Path(__file__).parent
    PARENT_DIR = THIS_DIR.parent

    def __init__(self, data_dir=None):
        # Set the download directory
        if data_dir:
            self.data_dir = pathlib.Path(str(data_dir))
        else:
            self.data_dir = self.PARENT_DIR.joinpath("data")

        # Initialize all the directories we will need
        if not self.data_dir.exists():
            self.data_dir.mkdir()
        self.raw_dir = self.data_dir.joinpath("raw")
        if not self.raw_dir.exists():
            self.raw_dir.mkdir()
        self.processed_dir = self.data_dir.joinpath("processed")
        if not self.processed_dir.exists():
            self.processed_dir.mkdir()

    def run(self):
        """
        Execute the downloader to fetch, prep and clean the source geotype files.
        """
        self.download()
        self.unzip()
        self.process()

    @property
    def zip_name(self):
        """
        The name of the source zipfile
        """
        raise NotImplementedError("All geotype subclasses must provide their own zip_name property.")

    @property
    def zip_path(self):
        """
        The full path to the source zipfile.
        """
        return self.raw_dir.joinpath(self.zip_name)

    @property
    def shp_name(self):
        """
        The name of the source shapefile.
        """
        return self.zip_name.replace(".zip", ".shp")

    @property
    def shp_path(self):
        """
        The full path to the source shapefile.
        """
        return self.raw_dir.joinpath(self.shp_name)

    @property
    def geojson_name(self):
        """
        The name of the target GeoJSON created by this downloader.
        """
        return f'{self.PROCESSED_NAME}.geojson'

    @property
    def geojson_path(self):
        """
        The full path to the target GeoJSON created by this downloader.
        """
        return self.processed_dir.joinpath(self.geojson_name)

    def download(self):
        """
        Download the source geotype files.
        """
        # Check if the zip file already exists
        if self.zip_path.exists():
            logger.debug(f"ZIP file already exists at {self.zip_path}")
            return

        # If it doesn't, download it from the Census site
        logger.debug(f"Downloading {self.url} to {self.zip_path}")
        urlretrieve(self.url, self.zip_path)
        time.sleep(1)

    def unzip(self):
        """
        Unzip the source geotype files.
        """
        # Check if the shape has already been unzipped
        if self.shp_path.exists():
            logger.debug(f"SHP already unzipped at {self.shp_path}")
            return

        # If not, unzip it now.
        logger.debug(f"Unzipping {self.zip_path} to {self.raw_dir}")
        with zipfile.ZipFile(self.zip_path, "r") as z:
            z.extractall(self.raw_dir)

    def process(self):
        """
        Refine the source data and convert to cleaned GeoJSON.
        """
        # Check if the geojson file already exists
        if self.geojson_path.exists():
            logger.debug(f"GeoJSON file already exists at {self.geojson_path}")
            return

        # Read in the source shapefile
        gdf = gpd.read_file(self.shp_path)

        # Trim it down to the subset of fields we want to keep
        trimmed = gdf[list(self.FIELD_CROSSWALK.keys())]

        # Rename the fields using the crosswalk
        cleaned = trimmed.rename(columns=self.FIELD_CROSSWALK)

        # Write out a GeoJSON file
        logger.debug(f"Writing out {len(cleaned)} shapes to {self.geojson_path}")
        cleaned.to_file(self.geojson_path, driver="GeoJSON")


class BaseStateDownloader(BaseDownloader):
    """
    A base downloader for a single state's source files.
    """
    def __init__(self, state, data_dir):
        # Configure the state
        self.state = us.states.lookup(state)
        super().__init__(data_dir)

    @property
    def geojson_name(self):
        return f"{self.PROCESSED_NAME}_{self.state.abbr.lower()}.geojson"


class BaseStateListDownloader(BaseDownloader):
    """
    A base downloader that will retrieve all 50 states.
    """
    def run(self):
        self.download()
        self.merge()
        self.process()

    @property
    def merged_path(self):
        """
        The location of all the source shapefiles merged together.
        """
        return self.raw_dir.joinpath(f"{self.PROCESSED_NAME}.shp")

    def download(self):
        # Loop through all the states and download the shapes
        for state in us.STATES:
            logger.debug(f"Downloading {state}")
            runner = self.DOWNLOADER_CLASS(state.abbr, data_dir=self.data_dir)
            runner.run()

    def merge(self):
        """
        Combine all the source shapefiles into a single shapefile.
        """
        if self.merged_path.exists():
            logger.debug(f"SHP file already exists at {self.merged_path}")
            return

        # Open all the shapes
        path_list = [
            self.DOWNLOADER_CLASS(state.abbr, data_dir=self.data_dir).shp_path
            for state in us.STATES
        ]
        df_list = [gpd.read_file(p) for p in path_list]

        # Concatenate them together
        df = gpd.pd.concat(df_list)

        logger.debug(f"Writing file with {len(df)} tracts to {self.merged_path}")
        df.to_file(self.merged_path, index=False)

    def process(self):
        """
        Process the raw data and convert it to our preferred format, GeoJSON.
        """
        if self.geojson_path.exists():
            logger.debug(f"GeoJSON file already exists at {self.geojson_path}")
            return

        gdf = gpd.read_file(self.merged_path)
        logger.debug(f"Writing out {len(gdf)} shapes to {self.geojson_path}")
        gdf.to_file(self.geojson_path, driver="GeoJSON")



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import collections
from census_map_downloader.base import BaseStateDownloader, BaseStateListDownloader

# Logging
import logging
logger = logging.getLogger(__name__)


class StatePlacesDownloader2018(BaseStateDownloader):
    """
    Download 2018 places for a single state.
    """
    YEAR = 2018
    PROCESSED_NAME = f"places_{YEAR}"
    # Page 47 https://www2.census.gov/geo/pdfs/maps-data/data/tiger/tgrshp2018/TGRSHP2018_TechDoc_Ch3.pdf
    FIELD_CROSSWALK = collections.OrderedDict({
        "STATEFP": "state_fips",
        "PLACEFP": "place_id",
        "GEOID": "geoid",
        "NAME": "place_name",
        "geometry": "geometry"
    })

    @property
    def url(self):
        return f"https://www2.census.gov/geo/tiger/TIGER{self.YEAR}/PLACE/tl_{self.YEAR}_{self.state.fips}_place.zip"

    @property
    def zip_name(self):
        return f"tl_{self.YEAR}_{self.state.fips}_place.zip"


class PlacesDownloader2018(BaseStateListDownloader):
    """
    Download all 2018 places in the United States.
    """
    YEAR = 2018
    PROCESSED_NAME = f"places_{YEAR}"
    DOWNLOADER_CLASS = StatePlacesDownloader2018



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import collections
from census_map_downloader.base import BaseStateDownloader, BaseStateListDownloader

# Logging
import logging
logger = logging.getLogger(__name__)


class StateBlocksDownloader2018(BaseStateDownloader):
    """
    Download 2018 blocks for a single state.
    """
    YEAR = 2018
    PROCESSED_NAME = f"blocks_{YEAR}"
    # Docs pg 14 (https://www2.census.gov/geo/pdfs/maps-data/data/tiger/tgrshp2018/TGRSHP2018_TechDoc_Ch3.pdf)
    FIELD_CROSSWALK = collections.OrderedDict({
        "STATEFP10": "state_fips",
        "COUNTYFP10": "county_fips",
        "BLOCKCE10": "block_id",
        "GEOID10": "geoid",
        "NAME10": "block_name",
        "geometry": "geometry"
    })

    @property
    def url(self):
        return f"https://www2.census.gov/geo/tiger/TIGER{self.YEAR}/TABBLOCK/tl_{self.YEAR}_{self.state.fips}_tabblock10.zip"

    @property
    def zip_name(self):
        return f"tl_{self.YEAR}_{self.state.fips}_tabblock10.zip"


class BlocksDownloader2018(BaseStateListDownloader):
    """
    Download all 2018 blocks in the United States.
    """
    DOWNLOADER_CLASS = StateBlocksDownloader2018

    def merge(self):
        """
        No merging on this downloader because it is too big.
        """
        pass

    def process(self):
        """
        No processing on this downloader because it is too big.
        """
        pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import collections
from census_map_downloader.base import BaseStateDownloader, BaseStateListDownloader

# Logging
import logging
logger = logging.getLogger(__name__)


class StateTractsDownloader2010(BaseStateDownloader):
    """
    Download 2010 tracts for a single state.
    """
    YEAR = 2010
    PROCESSED_NAME = f"tracts_{YEAR}"
    # Docs pg 57 (https://www2.census.gov/geo/pdfs/maps-data/data/tiger/tgrshp2018/TGRSHP2018_TechDoc_Ch3.pdf)
    FIELD_CROSSWALK = collections.OrderedDict({
        "STATEFP10": "state_fips",
        "COUNTYFP10": "county_fips",
        "TRACTCE10": "tract_id",
        "GEOID10": "geoid",
        "NAME10": "tract_name",
        "geometry": "geometry"
    })

    @property
    def url(self):
        return self.state.shapefile_urls("tract")

    @property
    def zip_name(self):
        return f"tl_2010_{self.state.fips}_tract10.zip"


class StateTractsDownloader2000(StateTractsDownloader2010):
    """
    Download 2000 tracts for a single state.
    """
    PROCESSED_NAME = "tracts_2000"
    # Docs pg 57 (https://www2.census.gov/geo/pdfs/maps-data/data/tiger/tgrshp2010/TGRSHP10SF1.pdf)
    FIELD_CROSSWALK = collections.OrderedDict({
        "STATEFP00": "state_fips",
        "COUNTYFP00": "county_fips",
        "TRACTCE00": "tract_id",
        "CTIDFP00": "geoid",
        "NAME10": "tract_name",
        "geometry": "geometry"
    })

    @property
    def url(self):
        return f"https://www2.census.gov/geo/tiger/TIGER2009/{self.zip_folder}/{self.zip_name}"

    @property
    def zip_name(self):
        return f"tl_2009_{self.state.fips}_tract00.zip"

    @property
    def zip_folder(self):
        return f"{self.state.fips}_{self.state.name.upper().replace(' ', '_')}"

    @property
    def geojson_name(self):
        return f"{self.PROCESSED_NAME}_{self.state.abbr.lower()}.geojson"


class TractsDownloader2010(BaseStateListDownloader):
    """
    Download all 2010 tracts in the United States.
    """
    YEAR = 2010
    PROCESSED_NAME = f"tracts_{YEAR}"
    DOWNLOADER_CLASS = StateTractsDownloader2010


class TractsDownloader2000(BaseStateListDownloader):
    """
    Download all 2000 tracts in the United States.
    """
    YEAR = 2000
    PROCESSED_NAME = f"tracts_{YEAR}"
    DOWNLOADER_CLASS = StateTractsDownloader2000



import os
from setuptools import setup


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()


setup(
    name='census-map-downloader',
    version='0.0.4',
    description="Easily download U.S. census maps",
    long_description=read('README.rst'),
    author='Los Angeles Times Data Desk',
    author_email='datadesk@latimes.com',
    url='http://www.github.com/datadesk/census-map-downloader',
    license="MIT",
    packages=(
        "census_map_downloader",
        "census_map_downloader.geotypes",
    ),
    install_requires=(
        "geopandas",
        "us",
        "click"
    ),
    entry_points="""
        [console_scripts]
        censusmapdownloader=census_map_downloader.cli:cmd
    """,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
    ],
    project_urls={
        'Maintainer': 'https://github.com/datadesk',
        'Source': 'https://github.com/datadesk/census-map-downloader',
        'Tracker': 'https://github.com/datadesk/census-map-downloader/issues'
    },
)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import collections
from census_map_downloader.base import BaseDownloader

# Logging
import logging
logger = logging.getLogger(__name__)


class CountiesDownloader2018(BaseDownloader):
    """
    Download counties.
    """
    PROCESSED_NAME = "counties_2018"
    # Docs pg 21 (https://www2.census.gov/geo/pdfs/maps-data/data/tiger/tgrshp2018/TGRSHP2018_TechDoc_Ch3.pdf)
    FIELD_CROSSWALK = collections.OrderedDict({
        "STATEFP": "state_fips",
        "COUNTYFP": "county_fips",
        "GEOID": "geoid",
        "NAME": "county_name",
        "geometry": "geometry"
    })

    @property
    def url(self):
        return "https://www2.census.gov/geo/tiger/TIGER2018/COUNTY/tl_2018_us_county.zip"

    @property
    def zip_name(self):
        return f"tl_2018_us_county.zip"
